import requests
import yaml
import json
import click


gen_host = 'http://bytepower-alb-01-1801127871.us-east-1.elb.amazonaws.com:9090'


def get_download_filename(language, options):
    if language == 'csharp':
        return '{}_{}.zip'.format(
                options['packageName'], 
                options['packageVersion'])
    else:
        raise RuntimeError('language {} is not supported.'.format(language))


def validate_specfile(specfile):
    validate_url = 'http://bytepower-alb-01-1801127871.us-east-1.elb.amazonaws.com:9091/debug'
    headers = {'content-type': 'application/json'}
    try:
        api_spec = yaml.load(open(specfile), Loader=yaml.FullLoader)
    except Exception:
        api_spec = json.load(open(specfile))
    resp = requests.post(validate_url, json=api_spec, headers=headers)
    data = resp.json()
    if data == {}:
        return True, None
    else:
        return False, data


@click.group(context_settings={'help_option_names': ['-h', '--help']})
def cli():
    pass


@cli.command()
@click.option('--specfile', '-s', help='API SPEC file name, in YAML or JSON format.')
def validate(specfile):
    if not specfile:
        print('Mssing options, see help below:')
        with click.Context(validate) as ctx:
            print(ctx.get_help())
        return
    print('validating spec file {}...'.format(specfile))
    is_valid, result = validate_specfile(specfile)
    if is_valid:
        print('spec file {} is valid.'.format(specfile))
    else:
        print('spec file {} is invalid.'.format(specfile))
        print('details are below:')
        print(result)


@cli.command()
@click.option('--language', '-l', default='csharp', help='Language of generated SDK, default is csharp.')
@click.option('--specfile', '-s', help='API SPEC file name, in YAML or JSON format.')
@click.option('--optionfile', '-o', help='Name of option file which has generation options in it, in JSON format.')
def generate(language, specfile, optionfile):
    if not all([language, specfile, optionfile]):
        print('Mssing options, see help below:')
        with click.Context(generate) as ctx:
            print(ctx.get_help())
        return
    print('validating spec file {}...'.format(specfile))
    is_valid, validate_result = validate_specfile(specfile)
    if is_valid:
        print('spec file {} is valid.'.format(specfile))
    else:
        print('spec file {} is invalid.'.format(specfile))
        print('details are below:')
        print(result)
    try:
        api_spec = yaml.load(open(specfile), Loader=yaml.FullLoader)
    except Exception:
        api_spec = json.load(open(specfile))
    headers = {'content-type': 'application/json'}
    options = json.loads(open(optionfile).read())
    data = {
        'spec': api_spec,
        'options': options
    }
    gen_sdk_api = '{}/api/gen/clients/{}'.format(gen_host, language)
    print('generating SDK...')
    resp = requests.post(gen_sdk_api, json=data, headers=headers)
    data = resp.json()
    download_url = '{}/api/gen/download/{}'.format(gen_host, data['code'])
    download_filename = get_download_filename(language, options)
    print('downloading SDK as file {}...'.format(download_filename))
    resp = requests.get(download_url)
    with open(download_filename, 'wb') as f:
        f.write(resp.content)
    print('SDK is generated in file {}'.format(download_filename))


if __name__ == '__main__':
    cli()
